package com.agilefaqs.rft.suggestions;

import static org.easymock.EasyMock.eq;
import static org.easymock.EasyMock.expect;
import static org.easymock.classextension.EasyMock.createMock;
import static org.easymock.classextension.EasyMock.replay;
import static org.easymock.classextension.EasyMock.reset;
import static org.easymock.classextension.EasyMock.verify;
import static org.junit.Assert.assertEquals;

import java.util.ArrayList;
import java.util.Calendar;

import org.junit.After;
import org.junit.Before;
import org.junit.Test;

import com.agilefaqs.rft.domain.Country;
import com.agilefaqs.rft.domain.LocationInformation;
import com.agilefaqs.rft.suggestions.IdentitySuggestionOneStrategy;
import com.agilefaqs.rft.suggestions.SuggestionParam;
import com.agilefaqs.rft.suggestions.strategies.ChildStrategyParam;
import com.agilefaqs.rft.suggestions.strategies.ChildSuggestionFactory;
import com.agilefaqs.rft.suggestions.strategies.ChildSuggestionStrategy;
import com.agilefaqs.rft.suggestions.strategies.RandomNumberBasedStrategy;
import com.agilefaqs.rft.suggestions.strategies.SimpleTokenStrategy;
import com.agilefaqs.rft.suggestions.strategies.YearBasedStrategy;

public class IdentitySuggestionOneStrategyTest {

    private static final String AT = "@";
    private static final String FIRST_NAME = "Naresh";
    private static final String LAST_NAME = "Jain";
    private static final String CURRENT_YEAR = String.valueOf(Calendar.getInstance().get(Calendar.YEAR));
    private final Country country = new Country("IN", "INDIA", "Indian");

    private final SuggestionParam params = new SuggestionParam(new LocationInformation(country, "mumbai"), FIRST_NAME, LAST_NAME);
    private final ChildStrategyParam childStrategyParam = new ChildStrategyParam(params.getValidFirstNameToken(), params
            .getValidLastNameToken(), null);
    private final SimpleTokenStrategy simpleTokenStrategy = createMock(SimpleTokenStrategy.class);
    private final RandomNumberBasedStrategy randomNumberBasedStrategy = createMock(RandomNumberBasedStrategy.class);
    private final YearBasedStrategy yearBasedStrategy = createMock(YearBasedStrategy.class);
    private final ChildSuggestionFactory childSuggestionFactory = createMock(ChildSuggestionFactory.class);
    private IdentitySuggestionOneStrategy strategyOne;
    private static final String RANDOM_NUMBER = "10001";
    final ArrayList<ChildSuggestionStrategy> childStrategies = new ArrayList<ChildSuggestionStrategy>() {
        {
            add(simpleTokenStrategy);
            add(yearBasedStrategy);
            add(randomNumberBasedStrategy);
        }
    };

    @Test
    public void shouldGenerateOfType_FirstName_AT_LastName_DOT_COM_IfAvailable() throws Exception {
        String suggestion = FIRST_NAME.concat(AT).concat(LAST_NAME).concat(".com");
        expect(simpleTokenStrategy.generateId(eq(childStrategyParam))).andReturn(suggestion);
        stopRecordingAndReplay();
        String Identity = strategyOne.getIdentity(params);
        assertEquals("expected was " + suggestion + " but was : " + Identity, Identity, suggestion);
    }

    @Test
    public void shouldGenerateOfType_FirstNameYear_AT_LastName_DOT_COM_IfFirstSuggestionIsUnavailable() throws Exception {
        String suggestion = FIRST_NAME.concat(CURRENT_YEAR).concat(AT).concat(LAST_NAME).concat(".com");
        expect(simpleTokenStrategy.generateId(eq(childStrategyParam))).andReturn(null);
        expect(yearBasedStrategy.generateId(eq(childStrategyParam))).andReturn(suggestion);
        stopRecordingAndReplay();
        String Identity = strategyOne.getIdentity(params);
        assertEquals("expected was " + suggestion + " but was : " + Identity, Identity, suggestion);

    }

    @Test
    public void shouldGenerateOfType_FirstNameRandomNumber_AT_LastName_DOT_COM_IfSecondSuggestionIsUnavailable() throws Exception {
        String suggestion = FIRST_NAME.concat(RANDOM_NUMBER).concat(AT).concat(LAST_NAME).concat(".com");
        expect(simpleTokenStrategy.generateId(eq(childStrategyParam))).andReturn(null);
        expect(yearBasedStrategy.generateId(eq(childStrategyParam))).andReturn(null);
        expect(randomNumberBasedStrategy.generateId(eq(childStrategyParam))).andReturn(suggestion);
        stopRecordingAndReplay();
        String Identity = strategyOne.getIdentity(params);
        assertEquals("expected was " + suggestion + " but was : " + Identity, Identity, suggestion);

    }

    @Before
    public void setUp() {
        reset(childSuggestionFactory);
        expect(childSuggestionFactory.getChildStrategiesForSuggestionOne()).andReturn(childStrategies);
        replay(childSuggestionFactory);
        strategyOne = new IdentitySuggestionOneStrategy(childSuggestionFactory);
        verify(childSuggestionFactory);
        reset(childSuggestionFactory, simpleTokenStrategy, randomNumberBasedStrategy, yearBasedStrategy);
    }

    public void stopRecordingAndReplay() {
        replay(simpleTokenStrategy, randomNumberBasedStrategy, yearBasedStrategy);
    }

    @After
    public void verifyMocks() {
        verify(simpleTokenStrategy, randomNumberBasedStrategy, yearBasedStrategy);
    }

}
