package com.agilefaqs.rft.suggestions;

import static org.easymock.EasyMock.createMock;
import static org.easymock.EasyMock.expect;
import static org.easymock.EasyMock.replay;
import static org.easymock.EasyMock.verify;
import static org.junit.Assert.assertEquals;

import java.util.Arrays;
import java.util.List;

import org.junit.Test;

import com.agilefaqs.rft.domain.Country;
import com.agilefaqs.rft.domain.DomainNameService;
import com.agilefaqs.rft.domain.LocationInformation;
import com.agilefaqs.rft.domain.UserService;
import com.agilefaqs.rft.suggestions.strategies.ChildSuggestionFactory;

public class PartialAcceptanceTest {
    private final Country country = new Country("IN", "India", "Indian");
    private final LocationInformation location = new LocationInformation(country, "Mumbai");
    private final UserService userService = createMock(UserService.class);
    private final DomainNameService domainNameService = createMock(DomainNameService.class);
    private final SuggestionsUtil utils = new SuggestionsUtil(domainNameService);
    private final RandomNumberGenerator randomNumberGenerator = new RandomNumberGenerator() {
        @Override
        public String next() {
            return "_random";
        }
    };
    private final ChildSuggestionFactory childSuggestionFactory = new ChildSuggestionFactory(userService, utils, randomNumberGenerator);
    private final SuggestionStrategyFactory suggestionsFactory = new SuggestionStrategyFactory(childSuggestionFactory);
    private final IdentityGenerator identityGenerator = new IdentityGenerator(suggestionsFactory);

    @Test
    public void generateIdsUsingNameLocationAndNationality() {
        expect(domainNameService.isCelebrityName("Naresh", "Jain")).andStubReturn(false);
        expect(domainNameService.validateFirstPartAndReturnRestrictedWordIfAny("Naresh")).andStubReturn(null);
        expect(domainNameService.validateSecondPartAndReturnRestrictedWordIfAny("Jain")).andStubReturn(null);
        expect(userService.isIdentityAvailable("naresh@jain.com")).andStubReturn(true);

        expect(domainNameService.isCelebrityName("Naresh", "India")).andStubReturn(false);
        expect(domainNameService.validateFirstPartAndReturnRestrictedWordIfAny("Naresh")).andStubReturn(null);
        expect(domainNameService.validateSecondPartAndReturnRestrictedWordIfAny("India")).andStubReturn(null);
        expect(userService.isIdentityAvailable("naresh@india.com")).andStubReturn(true);

        expect(domainNameService.isCelebrityName("Naresh", "Indian")).andStubReturn(false);
        expect(domainNameService.validateFirstPartAndReturnRestrictedWordIfAny("Naresh")).andStubReturn(null);
        expect(domainNameService.validateSecondPartAndReturnRestrictedWordIfAny("Indian")).andStubReturn(null);
        expect(userService.isIdentityAvailable("naresh@indian.com")).andStubReturn(true);

        expect(domainNameService.isCelebrityName("Naresh", "Mumbai")).andStubReturn(false);
        expect(domainNameService.validateFirstPartAndReturnRestrictedWordIfAny("Naresh")).andStubReturn(null);
        expect(domainNameService.validateSecondPartAndReturnRestrictedWordIfAny("Mumbai")).andStubReturn(null);
        expect(userService.isIdentityAvailable("naresh@mumbai.com")).andStubReturn(true);

        replay(userService, domainNameService);

        SuggestionParam suggestionParam = new SuggestionParam(location, "Naresh", "Jain");
        List<String> generatedIDs = identityGenerator.getGeneratedIDs(suggestionParam);
        List<String> expectedIds = ids("naresh@jain.com", "naresh@india.com", "naresh@indian.com", "naresh@mumbai.com");

        assertEquals(expectedIds, generatedIDs);

        verify(userService, domainNameService);
    }

    @Test
    public void avoidRestrictedWordsInIds() {
        expect(domainNameService.isCelebrityName("Naresh", "Jain")).andStubReturn(false);
        expect(domainNameService.validateFirstPartAndReturnRestrictedWordIfAny("Naresh")).andStubReturn("Naresh");

        expect(domainNameService.isCelebrityName("Nares", "Jain")).andStubReturn(false);
        expect(domainNameService.validateFirstPartAndReturnRestrictedWordIfAny("Nares")).andStubReturn(null);
        expect(domainNameService.validateSecondPartAndReturnRestrictedWordIfAny("Jain")).andStubReturn(null);
        expect(userService.isIdentityAvailable("nares@jain.com")).andStubReturn(true);

        expect(domainNameService.isCelebrityName("Nares", "India")).andStubReturn(false);
        expect(domainNameService.isCelebrityName("Naresh", "India")).andStubReturn(false);
        expect(domainNameService.validateFirstPartAndReturnRestrictedWordIfAny("Nares")).andStubReturn(null);
        expect(domainNameService.validateSecondPartAndReturnRestrictedWordIfAny("India")).andStubReturn(null);
        expect(userService.isIdentityAvailable("nares@india.com")).andStubReturn(true);

        expect(domainNameService.isCelebrityName("Nares", "Indian")).andStubReturn(false);
        expect(domainNameService.isCelebrityName("Naresh", "Indian")).andStubReturn(false);
        expect(domainNameService.validateFirstPartAndReturnRestrictedWordIfAny("Nares")).andStubReturn(null);
        expect(domainNameService.validateSecondPartAndReturnRestrictedWordIfAny("Indian")).andStubReturn(null);
        expect(userService.isIdentityAvailable("nares@indian.com")).andStubReturn(true);

        expect(domainNameService.isCelebrityName("Nares", "Mumbai")).andStubReturn(false);
        expect(domainNameService.isCelebrityName("Naresh", "Mumbai")).andStubReturn(false);
        expect(domainNameService.validateFirstPartAndReturnRestrictedWordIfAny("Nares")).andStubReturn(null);
        expect(domainNameService.validateSecondPartAndReturnRestrictedWordIfAny("Mumbai")).andStubReturn(null);
        expect(userService.isIdentityAvailable("nares@mumbai.com")).andStubReturn(true);

        replay(userService, domainNameService);

        SuggestionParam suggestionParam = new SuggestionParam(location, "Naresh", "Jain");
        List<String> generatedIDs = identityGenerator.getGeneratedIDs(suggestionParam);
        List<String> expectedIds = ids("nares@jain.com", "nares@india.com", "nares@indian.com", "nares@mumbai.com");

        assertEquals(expectedIds, generatedIDs);

        verify(userService, domainNameService);
    }

    @Test
    public void avoidCelebrityNamesInGeneratedIds() {
        expect(domainNameService.isCelebrityName("Naresh", "Jain")).andStubReturn(true);

        expect(domainNameService.isCelebrityName("Nares", "Jai")).andStubReturn(false);
        expect(domainNameService.validateFirstPartAndReturnRestrictedWordIfAny("Nares")).andStubReturn(null);
        expect(domainNameService.validateSecondPartAndReturnRestrictedWordIfAny("Jai")).andStubReturn(null);
        expect(userService.isIdentityAvailable("nares@jai.com")).andStubReturn(true);

        expect(domainNameService.isCelebrityName("Naresh", "India")).andStubReturn(false);
        expect(domainNameService.validateFirstPartAndReturnRestrictedWordIfAny("Naresh")).andStubReturn(null);
        expect(domainNameService.validateSecondPartAndReturnRestrictedWordIfAny("India")).andStubReturn(null);
        expect(userService.isIdentityAvailable("naresh@india.com")).andStubReturn(true);

        expect(domainNameService.isCelebrityName("Naresh", "Indian")).andStubReturn(false);
        expect(domainNameService.validateFirstPartAndReturnRestrictedWordIfAny("Naresh")).andStubReturn(null);
        expect(domainNameService.validateSecondPartAndReturnRestrictedWordIfAny("Indian")).andStubReturn(null);
        expect(userService.isIdentityAvailable("naresh@indian.com")).andStubReturn(true);

        expect(domainNameService.isCelebrityName("Naresh", "Mumbai")).andStubReturn(false);
        expect(domainNameService.validateFirstPartAndReturnRestrictedWordIfAny("Naresh")).andStubReturn(null);
        expect(domainNameService.validateSecondPartAndReturnRestrictedWordIfAny("Mumbai")).andStubReturn(null);
        expect(userService.isIdentityAvailable("naresh@mumbai.com")).andStubReturn(true);

        replay(userService, domainNameService);

        SuggestionParam suggestionParam = new SuggestionParam(location, "Naresh", "Jain");
        List<String> generatedIDs = identityGenerator.getGeneratedIDs(suggestionParam);
        List<String> expectedIds = ids("nares@jai.com", "naresh@india.com", "naresh@indian.com", "naresh@mumbai.com");

        assertEquals(expectedIds, generatedIDs);

        verify(userService, domainNameService);
    }

    @Test
    public void appendCurrentYearWithFirstNameIfIdIsNotAvailable() {
        expect(domainNameService.isCelebrityName("Naresh", "Jain")).andStubReturn(false);
        expect(domainNameService.validateFirstPartAndReturnRestrictedWordIfAny("Naresh")).andStubReturn(null);
        expect(domainNameService.validateSecondPartAndReturnRestrictedWordIfAny("Jain")).andStubReturn(null);
        expect(userService.isIdentityAvailable("naresh@jain.com")).andStubReturn(false);

        expect(domainNameService.isCelebrityName("Naresh2009", "Jain")).andStubReturn(false);
        expect(domainNameService.validateFirstPartAndReturnRestrictedWordIfAny("Naresh2009")).andStubReturn(null);
        expect(domainNameService.validateSecondPartAndReturnRestrictedWordIfAny("Jain")).andStubReturn(null);
        expect(userService.isIdentityAvailable("naresh2009@jain.com")).andStubReturn(true);

        expect(domainNameService.isCelebrityName("Naresh", "India")).andStubReturn(false);
        expect(domainNameService.validateFirstPartAndReturnRestrictedWordIfAny("Naresh")).andStubReturn(null);
        expect(domainNameService.validateSecondPartAndReturnRestrictedWordIfAny("India")).andStubReturn(null);
        expect(userService.isIdentityAvailable("naresh@india.com")).andStubReturn(true);

        expect(domainNameService.isCelebrityName("Naresh", "Indian")).andStubReturn(false);
        expect(domainNameService.validateFirstPartAndReturnRestrictedWordIfAny("Naresh")).andStubReturn(null);
        expect(domainNameService.validateSecondPartAndReturnRestrictedWordIfAny("Indian")).andStubReturn(null);
        expect(userService.isIdentityAvailable("naresh@indian.com")).andStubReturn(true);

        expect(domainNameService.isCelebrityName("Naresh", "Mumbai")).andStubReturn(false);
        expect(domainNameService.validateFirstPartAndReturnRestrictedWordIfAny("Naresh")).andStubReturn(null);
        expect(domainNameService.validateSecondPartAndReturnRestrictedWordIfAny("Mumbai")).andStubReturn(null);
        expect(userService.isIdentityAvailable("naresh@mumbai.com")).andStubReturn(true);

        replay(userService, domainNameService);

        SuggestionParam suggestionParam = new SuggestionParam(location, "Naresh", "Jain");
        List<String> generatedIDs = identityGenerator.getGeneratedIDs(suggestionParam);
        List<String> expectedIds = ids("naresh2009@jain.com", "naresh@india.com", "naresh@indian.com", "naresh@mumbai.com");

        assertEquals(expectedIds, generatedIDs);

        verify(userService, domainNameService);
    }

    @Test
    public void appendRandomNumberWithFirstNameIfIdIsNotAvailable() {
        expect(domainNameService.isCelebrityName("Naresh", "Jain")).andStubReturn(false);
        expect(domainNameService.validateFirstPartAndReturnRestrictedWordIfAny("Naresh")).andStubReturn(null);
        expect(domainNameService.validateSecondPartAndReturnRestrictedWordIfAny("Jain")).andStubReturn(null);
        expect(userService.isIdentityAvailable("naresh@jain.com")).andStubReturn(false);

        expect(domainNameService.isCelebrityName("Naresh2009", "Jain")).andStubReturn(false);
        expect(domainNameService.validateFirstPartAndReturnRestrictedWordIfAny("Naresh2009")).andStubReturn(null);
        expect(domainNameService.validateSecondPartAndReturnRestrictedWordIfAny("Jain")).andStubReturn(null);
        expect(userService.isIdentityAvailable("naresh2009@jain.com")).andStubReturn(false);

        expect(domainNameService.isCelebrityName("Naresh_random", "Jain")).andStubReturn(false);
        expect(domainNameService.validateFirstPartAndReturnRestrictedWordIfAny("Naresh_random")).andStubReturn(null);
        expect(domainNameService.validateSecondPartAndReturnRestrictedWordIfAny("Jain")).andStubReturn(null);
        expect(userService.isIdentityAvailable("naresh_random@jain.com")).andStubReturn(true);

        expect(domainNameService.isCelebrityName("Naresh", "India")).andStubReturn(false);
        expect(domainNameService.validateFirstPartAndReturnRestrictedWordIfAny("Naresh")).andStubReturn(null);
        expect(domainNameService.validateSecondPartAndReturnRestrictedWordIfAny("India")).andStubReturn(null);
        expect(userService.isIdentityAvailable("naresh@india.com")).andStubReturn(true);

        expect(domainNameService.isCelebrityName("Naresh", "Indian")).andStubReturn(false);
        expect(domainNameService.validateFirstPartAndReturnRestrictedWordIfAny("Naresh")).andStubReturn(null);
        expect(domainNameService.validateSecondPartAndReturnRestrictedWordIfAny("Indian")).andStubReturn(null);
        expect(userService.isIdentityAvailable("naresh@indian.com")).andStubReturn(true);

        expect(domainNameService.isCelebrityName("Naresh", "Mumbai")).andStubReturn(false);
        expect(domainNameService.validateFirstPartAndReturnRestrictedWordIfAny("Naresh")).andStubReturn(null);
        expect(domainNameService.validateSecondPartAndReturnRestrictedWordIfAny("Mumbai")).andStubReturn(null);
        expect(userService.isIdentityAvailable("naresh@mumbai.com")).andStubReturn(true);

        replay(userService, domainNameService);

        SuggestionParam suggestionParam = new SuggestionParam(location, "Naresh", "Jain");
        List<String> generatedIDs = identityGenerator.getGeneratedIDs(suggestionParam);
        List<String> expectedIds = ids("naresh_random@jain.com", "naresh@india.com", "naresh@indian.com", "naresh@mumbai.com");

        assertEquals(expectedIds, generatedIDs);

        verify(userService, domainNameService);
    }

    private List<String> ids(final String... ids) {
        return Arrays.asList(ids);
    }
}
